import {AfterViewInit, Component, ElementRef, Input, ViewChild} from "@angular/core";
import {AbstractComponent, CanvasUtil, GanttChart, Layer} from "schedule";

@Component({
  selector: "demo-schedule-button-bar-radar",
  templateUrl: "./demo-schedule-button-bar-radar.component.html",
  styleUrls: ["./demo-schedule-button-bar-radar.component.scss"]
})
export class DemoScheduleButtonBarRadarComponent extends AbstractComponent implements AfterViewInit {

  // Dimensions

  radarWidthPx: number = 300;
  radarHeightPx: number = 180;

  // Canvas

  @ViewChild("radarCanvas")
  private _radarCanvasElementRef: ElementRef<HTMLCanvasElement>;
  private _radarContext: CanvasRenderingContext2D;

  @ViewChild("radarAboveLayer")
  private _aboveLayerCanvasElementRef: ElementRef<HTMLCanvasElement>;
  private _aboveLayerContext: CanvasRenderingContext2D;

  radarActivityColor: string = "#96f97b";
  radarBackgroundColor: string = "black";
  radarTimelineAboveLayerColor: string = "rgba(255, 255, 255, 0.5)";

  // Gantt

  @Input()
  gantt: GanttChart;

  private _layers: Layer[];
  radarNavigation: boolean = false;
  loading: boolean = true;

  ngAfterViewInit(): void {
    super.ngAfterViewInit();
    setTimeout(() => {
      const graphics = this.gantt.getGraphics();
      this._layers = this.gantt.getLayers();
      this._radarCanvasElementRef.nativeElement.width = this.radarWidthPx;
      this._radarCanvasElementRef.nativeElement.height = this.radarHeightPx;
      this._radarContext = this._radarCanvasElementRef.nativeElement.getContext("2d");
      this._aboveLayerCanvasElementRef.nativeElement.width = this.radarWidthPx;
      this._aboveLayerCanvasElementRef.nativeElement.height = this.radarHeightPx + 20;
      this._aboveLayerContext = this._aboveLayerCanvasElementRef.nativeElement.getContext("2d");
      this.subscribe(graphics.getTimelineModelChangeObs(), () => this._drawCurrentTimeline());
      this._drawRadar();
      this._drawCurrentTimeline();
      this.loading = false;
    }, 200);
  }

  private _drawRadar(): void {
    const dateline = this.gantt.getTimeline().getDateline();
    const graphics = this.gantt.getGraphics();
    const temporalUnit = dateline.getPrimaryTemporalUnit();
    const zoneId = dateline.getZoneId();
    const rows = graphics.getTreeRows().length ? graphics.getTreeRows() : graphics.getRows();
    const latestTimeUsed = graphics.getLatestTimeUsed();
    const earliestTimeUsed = graphics.getEarliestTimeUsed();
    this._radarContext.fillStyle = this.radarBackgroundColor;
    this._radarContext.fillRect(0, 0, this.radarWidthPx, this.radarHeightPx);
    this._radarContext.strokeStyle = this.radarActivityColor;
    this._radarContext.lineWidth = 0.5;
    rows.forEach((row, rowNumber) => {
      const repository = row.getRepository();
      const earliestTimeUsedInRow = repository.getEarliestTimeUsed();
      const latestTimeUsedInRow = repository.getLatestTimeUsed();
      const y = 3 + (rowNumber / (rows.length - 1)) * (this.radarHeightPx - 6);
      if (earliestTimeUsedInRow && latestTimeUsedInRow) {
        this._layers.forEach(layer => {
          const activities = repository.getActivities(layer, earliestTimeUsedInRow, latestTimeUsedInRow, temporalUnit, zoneId);
          activities.forEach(activity => {
            const x1 = (activity.getStartTime() - earliestTimeUsed) / (latestTimeUsed - earliestTimeUsed) * this.radarWidthPx;
            const x2 = (activity.getEndTime() - earliestTimeUsed) / (latestTimeUsed - earliestTimeUsed) * this.radarWidthPx;
            CanvasUtil.strokeLine(this._radarContext, x1, y, x2, y);
          });
        });
      }
    });
  }

  private _drawCurrentTimeline(): void {
    const timeline = this.gantt.getTimeline();
    const graphics = this.gantt.getGraphics();
    const latestTimeUsed = graphics.getLatestTimeUsed();
    const earliestTimeUsed = graphics.getEarliestTimeUsed();
    const startTime = timeline.getVisibleStartTime() < earliestTimeUsed ? earliestTimeUsed : timeline.getVisibleStartTime();
    const endTime = timeline.getVisibleEndTime() > latestTimeUsed ? latestTimeUsed : timeline.getVisibleEndTime();
    const x = (startTime - earliestTimeUsed) / (latestTimeUsed - earliestTimeUsed) * this.radarWidthPx;
    const width = (endTime - startTime) / (latestTimeUsed - earliestTimeUsed) * this.radarWidthPx;
    this._aboveLayerContext.clearRect(0, 0, this.radarWidthPx, this.radarHeightPx + 20);
    this._aboveLayerContext.fillStyle = this.radarTimelineAboveLayerColor;
    this._aboveLayerContext.fillRect(x, 0, width < 1 ? 1 : width, this.radarHeightPx + 20);
  }

  navigateWithRadar(event: PointerEvent): void {
    if (this.radarNavigation) {
      const graphics = this.gantt.getGraphics();
      const timeline = this.gantt.getTimeline();
      const latestTimeUsed = graphics.getLatestTimeUsed();
      const earliestTimeUsed = graphics.getEarliestTimeUsed();
      const currentTimelineOffset = earliestTimeUsed - 0.5 * (timeline.getVisibleEndTime() - timeline.getVisibleStartTime());
      const location = currentTimelineOffset + (event.offsetX / this.radarWidthPx) * (latestTimeUsed - earliestTimeUsed);
      timeline.getModel().setStartTime(location);
      this.gantt.redraw();
    }
  }

}
